<?php

namespace Ulmus\Ldap\Entity\Field;

class LdapDatetime extends Datetime {

    public const DATETIME_EPOCH_DIFF = 11644477200;

    public function load(...$arguments)
    {
        if (is_numeric($arguments[0])) {
            return static::createDateTimeFromSystemTime($arguments[0]);
        }

        return parent::load(...$arguments);
    }

    # https://stackoverflow.com/a/64918390
    public static function createDateTimeFromSystemTime($time, $basis = '1601-01-01 00:00:00', $resolution = 1.E-7, $timeZone = 'UTC') : \DateTimeInterface
    {
        return new static((new \DateTime($basis.' UTC'))
            ->modify(round($time * $resolution ).' Seconds')
            ->setTimeZone(new \DateTimeZone($timeZone))
            ->format('Y-m-d H:i:s')
        );
    }

    public function save()
    {
        return (string) $this->getSystemTime();
    }

    public function getSystemTime($basis = '1601-01-01 00:00:00', $resolution = 1.E7, $timeZone = 'UTC') : int
    {
        $interval = (new \DateTime('1970-01-01 00:00:00 UTC'))->diff(new \DateTime($basis.' UTC'));

        $diff = $interval->days * 86400 + $interval->h * 3600 + $interval->i * 60 + $interval->s;

        return ( $this->setTimeZone(new \DateTimeZone($timeZone))->getTimestamp() + $diff ) * $resolution;
    }
}
